#!/usr/bin/env python

# fetch-sphinx.py: retrieve a particular Python version's stdlib list
# using its hosted Sphinx inventory.

import sys
import sphobjinv as soi

if __name__ == "__main__":
    vers = sys.argv[1]
    inv = soi.Inventory(url=f"https://docs.python.org/{vers}/objects.inv")
    modules = list(sorted(obj.name for obj in inv.objects if obj.role == "module"))
    print("\n".join(modules))



from importlib import resources

import pytest

import stdlib_list


@pytest.mark.parametrize(
    ("version", "canonicalized"),
    [("2.7", "2.7"), ("3.10", "3.10")],
)
def test_get_canonical_version(version, canonicalized):
    assert stdlib_list.get_canonical_version(version) == canonicalized


@pytest.mark.parametrize("version", ["nonsense", "1.2.3", "3.1000"])
def test_get_canonical_version_raises(version):
    with pytest.raises(ValueError, match=rf"No such version: {version}"):
        stdlib_list.get_canonical_version(version)


@pytest.mark.parametrize("version", [*stdlib_list.short_versions, *stdlib_list.long_versions])
def test_self_consistent(version):
    list_path = f"lists/{stdlib_list.get_canonical_version(version)}.txt"
    modules = resources.files("stdlib_list").joinpath(list_path).read_text().splitlines()

    for mod_name in modules:
        assert stdlib_list.in_stdlib(mod_name, version)

    assert modules == stdlib_list.stdlib_list(version)


@pytest.mark.parametrize(
    "version_file", [f.name for f in resources.files("stdlib_list").joinpath("lists").iterdir()]
)
def test_self_consistent_reverse(version_file):
    version = version_file.removesuffix(".txt")
    assert stdlib_list.stdlib_list(version)



#!/usr/bin/env python

import inspect
import pkgutil
import sys

SEEN_MODS = set()


def walk_pkgutil(mod_name, mod, io):
    for pkg in pkgutil.walk_packages(mod.__path__, mod_name + "."):
        if pkg.name in SEEN_MODS:
            continue
        else:
            # We don't recurse here because `walk_packages` takes care of
            # it for us.
            SEEN_MODS.add(pkg.name)
            print(pkg.name, file=io)


def walk_naive(mod_name, mod, io):
    for attr in dir(mod):
        attr_obj = getattr(mod, attr, None)
        # Shouldn't happen, but who knows.
        if attr_obj is None:
            continue

        # If this member isn't a module, skip it.
        if not inspect.ismodule(attr_obj):
            continue

        # To filter "real" submodules from re-exports, we try
        # and import the submodule by its qualified name.
        # If the import fails, we know it's a re-exported module.
        try:
            submod_name = mod_name + "." + attr
            __import__(submod_name)
            walk(submod_name, io)
        except ImportError:
            # ...but sometimes we do want to include re-exports, since
            # they might be things like "accelerator" modules that don't
            # appear anywhere else.
            # For example, `_bz2` might appear as a re-export.
            try:
                # Again, try and import to avoid module-looking object
                # that don't actually appear on disk. Experimentally,
                # there are a few of these (like "TK").
                __import__(attr)
                walk(attr, io)
            except ImportError:
                continue


def walk(mod_name, io):
    if mod_name in SEEN_MODS:
        return
    else:
        SEEN_MODS.add(mod_name)
        print(mod_name, file=io)

    # Try and import it.
    try:
        mod = __import__(mod_name)

        if hasattr(mod, "__path__"):
            walk_pkgutil(mod_name, mod, io)
        else:
            walk_naive(mod_name, mod, io)

    except ImportError:
        pass


if __name__ == "__main__":
    output = sys.argv[1]

    with open(output, mode="w") as io:
        for mod_name in sys.builtin_module_names:
            walk(mod_name, io)

        if hasattr(sys, "stdlib_module_names"):
            for mod_name in sys.stdlib_module_names:
                walk(mod_name, io)
        else:
            for mod_name in sys.stdin:
                # Our precomputed list might not start at the root, since it
                # might be a package rather than a module.
                if "." in mod_name:
                    top_mod = mod_name.split(".")[0]
                    walk(top_mod, io)
                walk(mod_name.rstrip("\n"), io)
